#include <string.h>
#include <tx_api.h>
#include "nagpio.h"
#include "ampire_utils.h"
#include "ampire_cfg.h"
#include "NA_ISR.h"

/*
 * El acceso a la lnea de RS para la seleccin comando/datos del LCD se tuvo que cambiar para ser usada
 * por GPIO (GPIO 103 presente en el header del display)dado que la lnea A1 (address 1) no presentaba un 
 * timing adecuado (ambos flancos eran coincidentes con el CS). 
 * */
static volatile unsigned short *cmd_ptr = (unsigned short *) 0x40000000;
static volatile unsigned short *data_ptr = (unsigned short *) 0x40000002;
//static volatile unsigned short *status_ptr = (unsigned short *) 0x40000004;
//static volatile unsigned short *read_ptr = (unsigned short *) 0x40000006;


static unsigned short storedImage[SCREEN_WIDTH * SCREEN_HEIGHT/*SCREEN_WIDTH*/];
TX_MUTEX myMutex;

void ampire_init(){
	tx_mutex_create(&myMutex, "edtMutex", TX_NO_INHERIT);
	return;
}

void getMutex(){
	tx_mutex_get(&myMutex, TX_WAIT_FOREVER);
	return;	
}

void releaseMutex(){
	tx_mutex_put(&myMutex);
	return;	
}


void write_cmd(unsigned char cmd)
{
		
	naGpioSetOutput(AMP_RS_PIN, 0);
    *cmd_ptr = cmd;
	naGpioSetOutput(AMP_RS_PIN, 1);    
    return;
}


unsigned short read_data(void)
{
    return *data_ptr;
}


void write_ram(unsigned short data)
{
	*data_ptr = data;             
    return;
}


void write_cmd_data(unsigned char cmd, unsigned short data)
{
	naGpioSetOutput(AMP_RS_PIN, 0);
    *cmd_ptr = cmd;
    naGpioSetOutput(AMP_RS_PIN, 1);
    
    *data_ptr = data;
	//*cmd_ptr = GCTRL_RAMRD;
    return;
}


void setxypos(unsigned int Left, unsigned int Right, unsigned int Top, unsigned int Bottom)
{
	naGpioSetOutput(AMP_RS_PIN, 0);
	*cmd_ptr  = 0x00;
	naGpioSetOutput(AMP_RS_PIN, 1);
	
	*data_ptr = (unsigned short) (Left >> 8)		;//REG_00
   	*data_ptr = (unsigned short) Left			;//REG_01
   	
   	*data_ptr = (unsigned short) (Right >> 8)	;//REG_02
   	*data_ptr =  (unsigned short)  Right			;//REG_03

   	*data_ptr = (unsigned short) (Top >> 8)		;//REG_04
   	*data_ptr = (unsigned short) Top				;//REG_05
   	
   	*data_ptr =  (unsigned short) (Bottom >> 8)	;//REG_06
   	*data_ptr = (unsigned short) Bottom		  	;//REG_07
   	//write_cmd(GCTRL_RAMRD);
	return;
}

void amp_write_data(unsigned int left, unsigned int right, unsigned int top, unsigned int bottom, unsigned short *color)
{
	int x, y, z = 0, len;
	unsigned short *pcolor;
	
	if(color == NULL)
		return;
	
	pcolor = color;

	if((left > (SCREEN_WIDTH - 1)) ||
        (top > (SCREEN_HEIGHT - 1)) ||
        (right < 0) ||
        (bottom < 0))
		return;

	if(right > (SCREEN_WIDTH - 1))
		right = (SCREEN_WIDTH - 1);

	if(bottom > (SCREEN_HEIGHT - 1))
		bottom = (SCREEN_HEIGHT - 1);

	len = (((right - left) + 1) * ((bottom - top) + 1));

	if(len == 0)
		return;
	 
	//tx_interrupt_control(TX_INT_DISABLE);  //Disable interrupt and work in peace

	//Set window
	setxypos(left, right, top, bottom);
	//Write Enable  for AMP Display	
	naGpioSetOutput(AMP_RS_PIN, 0);
	*cmd_ptr = GCTRL_RAMWR;
	naGpioSetOutput(AMP_RS_PIN, 1);
	
	//write image to EDT display	
 	for(x = 0; x < len; x++){
		*data_ptr = *pcolor++;			
	}
	
	//Write Disable  for AMP Display
	naGpioSetOutput(AMP_RS_PIN, 0);
	*cmd_ptr = GCTRL_RAMRD;
	naGpioSetOutput(AMP_RS_PIN, 1);
	
	//tx_interrupt_control(TX_INT_ENABLE); 	//Re-enable interrupts	
	
	
	//update pixels in static image
	for(y = top; y <= bottom; y++)
		for(x = left; x <= right; x++)
			storedImage[(y * SCREEN_WIDTH) + x] = color[z++];

	return;
}
int amp_GetStoredImage(unsigned short *image, int size)
{
	if(size < (SCREEN_HEIGHT * SCREEN_WIDTH))
		return -1;

    if(image == NULL)
        return -1;

	getMutex(); 

	memcpy(image, storedImage, (SCREEN_HEIGHT * SCREEN_WIDTH));

	releaseMutex(); 

	return 0;
}


unsigned short swap_short(unsigned short data)
{
	unsigned char *tData = (unsigned char *) &data;
	unsigned char buf[2];

	memcpy(buf, &data, 2);

	tData[0] = buf[1];
	tData[1] = buf[0];

	return data;
}


unsigned long swap_long(unsigned long data)
{
	unsigned char *tData = (unsigned char *) &data;
	unsigned char buf[4];

	memcpy(buf, &data, 4);

	tData[0] = buf[3];
	tData[1] = buf[2];
	tData[2] = buf[1];
	tData[3] = buf[0];

	return data;
}

