#include <string.h>
#include <tx_api.h>

#include "gpiomux_def.h"
#include "bsp_api.h"
#include "nagpio.h"

#include "spi_master_api.h"
#include "touch.h"

#define START_CMD                   (char) 0x00
#define READ_DATA                   (char) 0x00
#define READ_Y                      (char) 0x90
#define READ_X                      (char) 0xD0

#define EVENT_THREAD_STACK      4096
#define SPI_PORT                "/spi/0"

#define APP_SPI_MASTER_GPIO_CS      0
#define APP_SPI_MASTER_GPIO_CLK     5
#define APP_SPI_MASTER_GPIO_RXD     3
#define APP_SPI_MASTER_GPIO_TXD     7



static int touchpin = -1;
static AMP_touch_callback ampUserCallback;
static void *eventThreadStack = NULL;
static TX_THREAD eventThread;

static unsigned short calc_pos(unsigned short *pos)
{
	unsigned short values[3];
	unsigned short retval;

	values[0] = abs(pos[0] - pos[1]);
	values[1] = abs(pos[0] - pos[2]);
	values[2] = abs(pos[1] - pos[2]);

	if((values[0] < values[1]) && (values[0] < values[2]))
		retval = (pos[0] + pos[1]) / 2;

	else if(values[1] < values[2])
		retval = (pos[0] + pos[2]) / 2;

	else
		retval = (pos[1] + pos[2]) / 2;

	return retval;
}

static void event_thread(unsigned long unused)
{
	unsigned short x[3], y[3];
	int offset = 0, z, up = 0;
	unsigned int state;
	NaStatus retval;
	char command[8];
	char data[40] __attribute__ ((aligned(32))); //this alignment does not seem to work

	NA_UNUSED_PARAM(unused);

	command[0] = START_CMD;
	command[1] = READ_X;
	command[2] = READ_DATA;
	command[3] = READ_DATA;
	command[4] = START_CMD;
	command[5] = READ_Y;
	command[6] = READ_DATA;
	command[7] = READ_DATA;

	if((int) data % 32)
		offset = (32 - (int) data % 32);

	while(1)
	{
		tx_thread_sleep(NABspTicksPerSecond / 10);  //I found polling every 100ms was the way to go
		NAgetGPIOpin(touchpin, &state);   			//If I used an interrupt or polled at quicker rate

		if(state)
		{
			if(up)
			{
				up = 0;
				ampUserCallback(x[0], y[0], up); // was just released
			}

			continue;
		}

		for(z = 0; z < 3; z++)
		{
			retval = NASpiReadWrite(SPI_PORT, command, data + offset, 8);
			memcpy(&x[z], data + offset + 2, 2);
			x[z] = (x[z] * 320) / 32768;
			memcpy(&y[z], data + offset + 6, 2);
			y[z] = (y[z] * 240) / 32768;
			y[z] = abs(240 - y[z]);
		}

		x[0] = calc_pos(x);
		y[0] = calc_pos(y);

		if(up == 0)
		{
			up++;
			ampUserCallback(x[0], y[0], up); // initial press
		}

		else
		{
			up = 2;
			ampUserCallback(x[0], y[0], up); // held down
		}
	}

	return;
}


int amp_setupTouch(void *callback, int touch)
{
	static int initialized = 0;
    int retval;
	NASpiDeviceConfigType myDevice;

	if(initialized)
		return 0;

	if(callback == NULL)
		return AMP_BAD_CALLBACK;

	if(naGpioSetInput(touch) != 1)
		return AMP_BAD_TOUCH_PIN;

	if(NAconfigureGPIOpin(APP_SPI_MASTER_GPIO_CS, NA_GPIO_FUNC4_STATE, 0))
		return AMP_BAD_SPI_CS;

	if(NAconfigureGPIOpin(APP_SPI_MASTER_GPIO_CLK, NA_GPIO_FUNC4_STATE, 0))
		return AMP_BAD_SPI_CLK;

	if(NAconfigureGPIOpin(APP_SPI_MASTER_GPIO_RXD, NA_GPIO_FUNC4_STATE, 0))
		return AMP_BAD_SPI_RXD;

	if(NAconfigureGPIOpin(APP_SPI_MASTER_GPIO_TXD, NA_GPIO_FUNC4_STATE, 0))
		return AMP_BAD_SPI_TXD;

	myDevice.port = 0;
	myDevice.csAssertFn = NULL;
	myDevice.csDessertFn = NULL;
	myDevice.spiMode = NA_SPI_MODE0;
	myDevice.bitOrder = NA_SPI_MSB;
	myDevice.baudRate = 300000;
	myDevice.deselectTime = 10;
	strcpy(myDevice.name, SPI_PORT);

	retval = NASpiRegisterDevice(&myDevice);

	if(retval != NASTATUS_SUCCESS)
		return AMP_UNABLE_TO_REGISTER_SPI;

	eventThreadStack = (void *) malloc (EVENT_THREAD_STACK);

	if(eventThreadStack == NULL)
		return AMP_MALLOC_ERROR;

	retval = tx_thread_create(&eventThread,
		"SPITouchEventThread",
		event_thread,
		0,
		eventThreadStack,
		EVENT_THREAD_STACK,
		16,
		16,
		1,
		1);

	if(retval != 0)
		return AMP_THREAD_ERROR;

	ampUserCallback = callback;
	touchpin = touch;

	initialized++;

	return 0;
}





